"""Test for the template tags of the ``review`` app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from mixer.backend.django import mixer

from ..templatetags import review_tags
from .test_app.models import WeatherCondition


class GetReviewsTestCase(TestCase):
    """Tests for the ``get_reviews`` template tag."""
    longMessage = True

    def setUp(self):
        self.reviewed_item = mixer.blend('test_app.WeatherCondition')
        self.review = mixer.blend(
            'review.Review',
            content_type=ContentType.objects.get_for_model(WeatherCondition),
            reviewed_item=self.reviewed_item)

    def test_tag(self):
        self.assertEqual(len(review_tags.get_reviews(self.reviewed_item)), 1)
        self.review.delete()
        self.assertEqual(len(review_tags.get_reviews(self.reviewed_item)), 0)


class GetReviewAverageTestCase(TestCase):
    """Tests for the ``get_review_average`` template tag."""
    longMessage = True

    def setUp(self):
        self.reviewed_item = mixer.blend('test_app.WeatherCondition')

    def test_tag(self):
        self.assertFalse(review_tags.get_review_average(self.reviewed_item))
        review = mixer.blend(
            'review.Review', reviewed_item=self.reviewed_item,
            content_type=ContentType.objects.get_for_model(WeatherCondition),
            object_id=self.reviewed_item.pk,
        )
        self.assertEqual(review_tags.get_review_average(self.reviewed_item), 0)
        rating = mixer.blend('review.Rating', review=review)
        mixer.blend('review.Rating', review=review, category=rating.category)
        """
        Fix those tests!
        self.assertEqual(review_tags.get_review_average(self.reviewed_item), 3)

        """


class GetReviewCountTestCase(TestCase):
    """Tests for the ``get_review_count`` template tag."""
    longMessage = True

    def setUp(self):
        self.reviewed_item = mixer.blend('test_app.WeatherCondition')
        self.review = mixer.blend(
            'review.Review',
            content_type=ContentType.objects.get_for_model(WeatherCondition),
            reviewed_item=self.reviewed_item)

    def test_tag(self):
        self.assertEqual(
            review_tags.get_review_count(self.reviewed_item), 1)
        self.review.delete()
        self.assertEqual(
            review_tags.get_review_count(self.reviewed_item), 0)


class RenderCategoryAveragesTestCase(TestCase):
    """Tests for the ``render_category_averages`` template tag."""
    longMessage = True

    def setUp(self):
        self.reviewed_item = mixer.blend('test_app.WeatherCondition')
        self.review = mixer.blend(
            'review.Review',
            content_type=ContentType.objects.get_for_model(WeatherCondition),
            reviewed_item=self.reviewed_item)

    def test_tag(self):
        expected_value = {
            'reviewed_item': self.reviewed_item,
            'category_averages': {},
        }
        self.assertEqual(
            review_tags.render_category_averages(self.reviewed_item, 5),
            expected_value)

        self.rating = mixer.blend(
            'review.Rating', review=self.review, value='2')
        expected_value = {
            'reviewed_item': self.reviewed_item,
            'category_averages': {self.rating.category: 2.0},
        }
        self.assertEqual(
            review_tags.render_category_averages(self.reviewed_item, 5),
            expected_value)

        expected_value = {
            'reviewed_item': self.reviewed_item,
            'category_averages': {self.rating.category: 40.0},
        }
        self.assertEqual(
            review_tags.render_category_averages(self.reviewed_item, 100),
            expected_value)

    def test_tag_more_extensively(self):
        self.review = mixer.blend(
            'review.Review', reviewed_item=self.reviewed_item,
            object_id=self.reviewed_item.pk,
            content_type=ContentType.objects.get_for_model(WeatherCondition))
        # the test_tag case was merely to cover some basic functionality. This
        # goes more in depth.
        rating1 = mixer.blend('review.Rating', review=self.review, value='4')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 5):
            mixer.blend('review.RatingCategoryChoiceTranslation',
                        language_code='en-us',
                        ratingcategory=rating1.category, value=i)
        rating2 = mixer.blend('review.Rating', review=self.review, value='4')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 7):
            mixer.blend('review.RatingCategoryChoiceTranslation',
                        language_code='en-us',
                        ratingcategory=rating2.category, value=i)
        mixer.blend('review.Rating', category=rating2.category,
                    review=self.review, value=None)
        mixer.blend('review.Rating', category=rating2.category,
                    review=self.review, value=None)

        """
        Fix those tests!
        expected_value = {
            'reviewed_item': self.reviewed_item,
            'category_averages': {
                rating1.category: 6.0,
                rating2.category: 4.0,
            },
        }

        self.assertEqual(
            review_tags.render_category_averages(self.reviewed_item, 6),
            expected_value,
        )

        """


class TotalReviewAverageTestCase(TestCase):
    """Tests for the ``total_review_average`` template tag."""
    longMessage = True

    def setUp(self):
        self.content_object = mixer.blend('test_app.WeatherCondition')
        self.review = mixer.blend(
            'review.Review', reviewed_item=self.content_object,
            content_type=ContentType.objects.get_for_model(WeatherCondition),
        )
        self.rating1 = mixer.blend('review.Rating', review=self.review,
                                   value='4')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 5):
            mixer.blend('review.RatingCategoryChoiceTranslation',
                        language_code='en-us',
                        ratingcategory=self.rating1.category, value=i)
        self.rating2 = mixer.blend('review.Rating', review=self.review,
                                   value='6')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 7):
            mixer.blend('review.RatingCategoryChoiceTranslation',
                        language_code='en-us',
                        ratingcategory=self.rating2.category, value=i)

    def test_tag(self):
        self.assertEqual(
            review_tags.total_review_average(self.content_object), 100)
        mixer.blend('review.Rating', category=self.rating1.category,
                    review=self.review, value='0')
        mixer.blend('review.Rating', category=self.rating2.category,
                    review=self.review, value='0')
        self.assertEqual(
            review_tags.total_review_average(self.content_object), 50)
        mixer.blend('review.Rating', category=self.rating1.category,
                    review=self.review, value='')
        mixer.blend('review.Rating', category=self.rating2.category,
                    review=self.review, value='')
        self.assertEqual(
            review_tags.total_review_average(self.content_object), 50)
        self.assertEqual(
            review_tags.total_review_average(self.content_object, 10), 5)
        self.assertEqual(
            review_tags.total_review_average(self.content_object, 5), 2.5)


class UserHasReviewedTestCase(TestCase):
    """Tests for the ``user_has_reviewed`` template tag."""
    longMessage = True

    def setUp(self):
        self.user = mixer.blend('auth.User')
        self.content_object = mixer.blend('test_app.WeatherCondition')
        self.review = mixer.blend(
            'review.Review', user=self.user, reviewed_item=self.content_object,
            content_type=ContentType.objects.get_for_model(WeatherCondition),
        )
        self.other_user = mixer.blend('auth.User')

    def test_tag(self):
        self.assertTrue(
            review_tags.user_has_reviewed(self.content_object, self.user))
        self.assertFalse(
            review_tags.user_has_reviewed(self.content_object,
                                          self.other_user))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-12 15:03
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Rating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(blank=True, choices=[(b'5', b'5'), (b'4', b'4'), (b'3', b'3'), (b'2', b'2'), (b'1', b'1')], max_length=20, null=True, verbose_name='Value')),
            ],
            options={
                'ordering': ['category', 'review'],
            },
        ),
        migrations.CreateModel(
            name='RatingCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.SlugField(blank=True, max_length=32, verbose_name='Identifier')),
                ('counts_for_average', models.BooleanField(default=True, verbose_name='Counts for average rating')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RatingCategoryChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(blank=True, max_length=20, null=True, verbose_name='Value')),
                ('ratingcategory', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='choices', to='review.RatingCategory', verbose_name='Rating category')),
            ],
            options={
                'ordering': ('-value',),
            },
        ),
        migrations.CreateModel(
            name='RatingCategoryChoiceTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=128, verbose_name='Label')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='review.RatingCategoryChoice')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'review_ratingcategorychoice_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='RatingCategoryTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256)),
                ('question', models.CharField(blank=True, max_length=512, null=True)),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='review.RatingCategory')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'review_ratingcategory_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Review',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('content', models.TextField(blank=True, max_length=1024, verbose_name='Content')),
                ('language', models.CharField(blank=True, max_length=5, verbose_name='Language')),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('average_rating', models.FloatField(default=0, verbose_name='Average rating')),
                ('extra_object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('extra_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reviews_attached', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ['-creation_date'],
            },
        ),
        migrations.CreateModel(
            name='ReviewExtraInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(max_length=256, verbose_name='Type')),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('review', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='review.Review', verbose_name='Review')),
            ],
            options={
                'ordering': ['type'],
            },
        ),
        migrations.AddField(
            model_name='rating',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='review.RatingCategory', verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='rating',
            name='review',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ratings', to='review.Review', verbose_name='Review'),
        ),
        migrations.AlterUniqueTogether(
            name='ratingcategorytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='ratingcategorychoicetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]



try:
    from django.contrib.auth import get_user_model
except ImportError:  # Django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()


USER_MODEL = {
    'orm_label': '%s.%s' % (User._meta.app_label, User._meta.object_name),
    'model_label': '%s.%s' % (User._meta.app_label, User._meta.module_name),
    'object_name': User.__name__,
}
